@@ -1,4 +1,4 @@
-//  $Id: RtConfig.cc,v 4.9 2002/04/09 11:48:45 katie Exp $
+//  $Id: RtConfig.cc,v 4.11 2002/06/28 13:18:48 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -207,6 +207,9 @@ void init_and_set_options (int argc, char **argv, char **envp) {
      {"-cisco_force_tilda", ARGV_BOOL, 
       (char *) NULL, (char *) &CiscoConfig::forceTilda,
       "Interpret * and + as ~* and ~+ operators in AS paths to shorten as path acl's lines.\n\t\t\t\tCisco only."},
+     {"-cisco_empty_lists", ARGV_BOOL, 
+      (char *) NULL, (char *) &CiscoConfig::emptyLists,
+      "Generate access lists for ANY and NOT ANY prefix filters.\n\t\t\t\tCisco only."},
      
      {"-junos_no_compress_acls", ARGV_BOOL, 
       (char *) NULL, (char *) &JunosConfig::compressAcls,
@@ -280,22 +283,31 @@ void RtConfig::loadDictionary() {
 // @RtConfig configureRouter rtr.isp.net
 void RtConfig::configureRouter(char *name) {
    const InetRtr *rtr = irr->getInetRtr(name);
-   const AttrProtocol *bgp = schema.searchProtocol("BGP4");
+   if (!rtr)	{
+	cerr << "Error: no object for router: " << name << endl;
+	return;
+   }
 
-   AttrIterator<AttrIfAddr> itr1(rtr, "ifaddr");
-   IPAddr myIP(itr1()->ifaddr.get_ipaddr());
+   const AttrProtocol *bgp = schema.searchProtocol("BGP4");
 
    AttrGenericIterator<ItemASNO> itr2(rtr, "local-as");
    ASt myAS = itr2()->asno;
+	
+   // fixed for multiple "ifaddr" attribute 
+   AttrIterator<AttrIfAddr> itr1(rtr, "ifaddr");
 
    AttrIterator<AttrPeer> itr(rtr, "peer");
 
    for (const AttrPeer *peer = itr.first(); peer; peer = itr.next()) {
-      if (peer->protocol == bgp) {
-	 ASt peerAS = ((ItemASNO *) peer->searchOption("asno")->args->head())->asno;
-	 importP(myAS, &myIP, peerAS, peer->peer);
-	 exportP(myAS, &myIP, peerAS, peer->peer);
-      }
+	for (const AttrIfAddr *ifaddr = itr1.first(); ifaddr; ifaddr = itr1.next()) 
+	{
+       		IPAddr myIP(ifaddr->ifaddr.get_ipaddr());
+      		if (peer->protocol == bgp) {
+	 		ASt peerAS = ((ItemASNO *) peer->searchOption("asno")->args->head())->asno;
+	 		importP(myAS, &myIP, peerAS, peer->peer);
+	 		exportP(myAS, &myIP, peerAS, peer->peer);
+      		}
+ 	}
    }
 }
 
@@ -311,10 +323,26 @@ void RtConfig::printPrefixes(char *filter, char *fmt) {
       
       NormalExpression *ne = 
 	 NormalExpression::evaluate(itr()->filter, ~0);
+
+      if (ne && ne->is_any() != NEITHER)
+        cerr << "Warning: filter matches ANY/NOT ANY" << endl;
+
       if (ne && ne->singleton_flag == NormalTerm::PRFX)
 	 printPrefixes_(ne->first()->prfx_set, fmt);
       else
-	 cerr << "Error: Filter can only contain prefix filters." << endl;
+	 if (ne && ne->is_any() != NEITHER )
+         {
+		if (ciscoConfig.emptyLists)	
+		{
+                	// print "any/not any" prefix list - changed by katie@ripe.net
+                	SetOfPrefix *nets = new SetOfPrefix;
+                	if (ne->is_any() == ANY)
+                        	nets->not_ = true;
+                	printSuperPrefixRanges_(*nets,fmt);
+		}
+         }
+        else
+	 	cerr << "Error: Badly formed prefix filter." << endl;
       
       delete ne;
    }
@@ -337,8 +365,16 @@ void RtConfig::printPrefixRanges(char *filter, char *fmt) {
       if (ne && ne->singleton_flag == NormalTerm::PRFX)
 	 printPrefixRanges_(ne->first()->prfx_set, fmt);
       else
-	 cerr << "Error: Filter can only contain prefix filters." << endl;
-      
+	if (ne && ne->is_any() != NEITHER )
+         {
+                // print "any/not any" prefix list - changed by katie@ripe.net
+                SetOfPrefix *nets = new SetOfPrefix;
+                if (ne->is_any() == ANY)
+                        nets->not_ = true;
+                printSuperPrefixRanges_(*nets,fmt);
+         }
+	else
+     		cerr << "Error: Badly formed prefix filter." << endl; 
       delete ne;
    }
    
@@ -357,10 +393,26 @@ void RtConfig::printSuperPrefixRanges(char *filter, char *fmt) {
       
       NormalExpression *ne = 
 	 NormalExpression::evaluate(itr()->filter, ~0);
+
+      if (ne && ne->is_any() != NEITHER)
+        cerr << "Warning: filter matches ANY/NOT ANY" << endl;
+
       if (ne && ne->singleton_flag == NormalTerm::PRFX)
 	 printSuperPrefixRanges_(ne->first()->prfx_set, fmt);
       else
-	 cerr << "Error: Filter can only contain prefix filters." << endl;
+	 if (ne && ne->is_any() != NEITHER)
+         {
+		if (ciscoConfig.emptyLists)	
+		{
+                	// print "any/not any" prefix list - changed by katie@ripe.net
+                	SetOfPrefix *nets = new SetOfPrefix;
+                	if (ne->is_any() == ANY)
+                        	nets->not_ = true;
+                	printSuperPrefixRanges_(*nets,fmt);
+		}
+         }
+         else
+	 	cerr << "Error: Badly formed prefix filter." << endl;
       
       delete ne;
    }
@@ -584,11 +636,27 @@ void RtConfig::accessList(char *filter) {
       
       NormalExpression *ne = 
 	 NormalExpression::evaluate(itr()->filter, ~0);
+
+      if (ne && ne->is_any() != NEITHER)
+      	cerr << "Warning: filter matches ANY/NOT ANY" << endl;
+
       if (ne && ne->singleton_flag == NormalTerm::PRFX)
 	 printAccessList(ne->first()->prfx_set);
-      else
-	 cerr << "Error: Filter can only contain prefix filters." << endl;
-      
+      else 
+	if (ne && ne->is_any() != NEITHER )
+	 {
+		if (ciscoConfig.emptyLists)
+		{
+			// print "any/not any" prefix list - changed by katie@ripe.net
+      			SetOfPrefix *nets = new SetOfPrefix;
+			if (ne->is_any() == ANY) 
+				nets->not_ = true;
+      			printAccessList(*nets);	
+	 	}
+	  }
+	else 
+		cerr << "Error: Badly formed prefix filter." << endl;
+
       delete ne;
    }
    
@@ -607,16 +675,27 @@ void RtConfig::aspathAccessList(char *filter) {
       
       NormalExpression *ne = 
 	 NormalExpression::evaluate(itr()->filter, ~0);
+
+      if (ne && ne->is_any() != NEITHER)
+        cerr << "Warning: filter matches ANY/NOT ANY" << endl;
+
       if (ne && ne->singleton_flag == NormalTerm::AS_PATH)
 	 printAspathAccessList(ne->first()->as_path);
       else
-	 cerr << "Error: Filter can only contain as path filters." << endl;
-      
+	cerr << "Error: badly formed AS_path filter." << endl;
+
       delete ne;
    }
    
    delete o;
 }
 
+void RtConfig::printPolicyWarning(ASt as, IPAddr* addr, ASt peerAS, IPAddr* peerAddr, const char* policy)	
+{
+	cerr << "Warning: AS" << as;
+        cerr << " has no " << policy << " policy for AS" << peerAS;
+        cerr << " " << peerAddr->get_text();
+        cerr << " at " << addr->get_text() << endl;
+}
 
 
