@@ -1,4 +1,4 @@
-//  $Id: f_cisco.cc,v 4.9 2002/04/09 11:48:46 katie Exp $
+//  $Id: f_cisco.cc,v 4.11 2002/06/28 13:19:18 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -76,6 +76,7 @@ bool CiscoConfig::compressAcls = true;
 bool CiscoConfig::usePrefixLists = false;
 bool CiscoConfig::eliminateDupMapParts = false;
 bool CiscoConfig::forceTilda = false;
+bool CiscoConfig::emptyLists = false;
 int  CiscoConfig::mapIncrements = 1;
 int  CiscoConfig::mapCount = 1;
 int  CiscoConfig::mapNumbersStartAt = 1;
@@ -864,13 +865,46 @@ int CiscoConfig::print(NormalExpression *ne, PolicyActionList *actn,
    static ListOf2Ints empty_list(1);
 
    Debug(Channel(DBG_CISCO) << "# ne: " << *ne << "\n");
+   
+   // modified by katie@ripe.net to handle any/not any expr
+   // for prefix filters, cisco only
 
-   if (ne->isEmpty()) 
-      return last;
-
-   if (ne->is_universal()) 
-      last = 1;
+   if (ne->is_any() != NEITHER)	
+	cerr << "Warning: filter matches ANY/NOT ANY" << endl;
 
+   if (ne->isEmpty())
+   {
+	if (ne->singleton_flag == -1)
+	{
+		if (emptyLists)
+		{
+			// generate an empty filter 
+                	FilterOfPrefix *fltr = (FilterOfPrefix *) new SetOfPrefix;
+                	NormalTerm *nt = new NormalTerm;
+                	nt->prfx_set = *fltr;
+                	nt->make_universal(NormalTerm::PRFX);
+                	*ne += nt;
+		}
+	}
+	else
+		return last;
+   }
+   if (ne->is_universal())
+   {
+	if (ne->singleton_flag == -1)
+	{
+		if (emptyLists)
+		{
+			// generate empty filter and negate to enable 'permit'
+                	FilterOfPrefix *fltr = (FilterOfPrefix *) new SetOfPrefix;
+                	fltr->not_ = true;
+                	ne->first()->prfx_set = *fltr;
+		}
+	}
+	else
+		last = 1;
+   }
+	
    bool needToPrintNoRouteMap = false;
    if (strcmp(mapName, lastMapName)) {
       strcpy(lastMapName, mapName);
@@ -983,10 +1017,11 @@ void CiscoConfig::exportP(ASt asno, IPAddr *addr,
    AutNumSelector<AttrExport> itr(autnum, "export", 
 				  NULL, peerAS, peer_addr, addr);
    const FilterAction *fa = itr.first();
-   if (! fa)
-      cerr << "Warning: AS" << asno 
-	   << " has no export policy for AS" << peerAS << endl;
-
+   if (! fa) {
+      printPolicyWarning(asno, addr, peerAS, peer_addr, "export");
+      return;
+   }
+   
    NormalExpression *ne;
    NormalExpression done;
    int last = 0;
@@ -1030,14 +1065,16 @@ void CiscoConfig::importP(ASt asno, IPAddr *addr,
     }
 
    int preAclID = prefixMgr.lastID();
+
    // get matching import attributes
    AutNumSelector<AttrImport> itr(autnum, "import", 
 				  NULL, peerAS, peer_addr, addr);
    const FilterAction *fa = itr.first();
-   if (! fa)
-      cerr << "Warning: AS" << asno 
-	   << " has no import policy for AS" << peerAS << endl;
-
+   if (! fa)	{
+      printPolicyWarning(asno, addr, peerAS, peer_addr, "import");
+      return;
+   }
+   
    NormalExpression *ne;
    NormalExpression done;
    int last = 0;
@@ -1280,8 +1317,7 @@ void CiscoConfig::outboundPacketFilter(char *ifname, ASt as, IPAddr* addr,
 				  NULL, peerAS,peerAddr, addr);
    const FilterAction *fa = itr.first();
    if (! fa)
-      cerr << "Warning: AS" << as 
-	   << " has no export policy for AS" << peerAS << endl;
+      printPolicyWarning(as, addr, peerAS, peerAddr, "export");
 
    NormalExpression *ne;
    int last = 0;
